@@ -1,5 +1,5 @@
 /*
- *   $Id: interface.c,v 1.13 2005/12/30 16:12:23 psavola Exp $
+ *   $Id: interface.c,v 1.18 2006/08/24 11:41:39 psavola Exp $
  *
  *   Authors:
  *    Lars Fenneberg		<lf@elemental.net>	 
@@ -23,6 +23,7 @@ iface_init_defaults(struct Interface *iface)
 {
 	memset(iface, 0, sizeof(struct Interface));
 
+	iface->HasFailed	  = 0;
 	iface->IgnoreIfMissing	  = DFLT_IgnoreIfMissing;
 	iface->AdvSendAdvert	  = DFLT_AdvSendAdv;
 	iface->MaxRtrAdvInterval  = DFLT_MaxRtrAdvInterval;
@@ -38,10 +39,10 @@ iface_init_defaults(struct Interface *iface)
 	iface->MinDelayBetweenRAs   = DFLT_MinDelayBetweenRAs;
 	iface->AdvMobRtrSupportFlag = DFLT_AdvMobRtrSupportFlag;
 
-	iface->MinRtrAdvInterval = DFLT_MinRtrAdvInterval(iface);
-	iface->AdvDefaultLifetime = DFLT_AdvDefaultLifetime(iface);
+	iface->MinRtrAdvInterval = -1;
+	iface->AdvDefaultLifetime = -1;
 	iface->AdvDefaultPreference = DFLT_AdvDefaultPreference;
-	iface->HomeAgentLifetime = DFLT_HomeAgentLifetime(iface);
+	iface->HomeAgentLifetime = -1;
 }
 
 void
@@ -67,6 +68,17 @@ route_init_defaults(struct AdvRoute *route, struct Interface *iface)
 	route->AdvRoutePreference = DFLT_AdvRoutePreference;
 }
 
+void
+rdnss_init_defaults(struct AdvRDNSS *rdnss, struct Interface *iface)
+{
+	memset(rdnss, 0, sizeof(struct AdvRDNSS));
+		
+	rdnss->AdvRDNSSPreference = DFLT_AdvRDNSSPreference;
+	rdnss->AdvRDNSSOpenFlag = DFLT_AdvRDNSSOpenFlag;
+	rdnss->AdvRDNSSLifetime = DFLT_AdvRDNSSLifetime(iface);
+	rdnss->AdvRDNSSNumber = 0;
+}
+
 int
 check_iface(struct Interface *iface)
 {
@@ -93,6 +105,9 @@ check_iface(struct Interface *iface)
 		prefix = prefix->next;
 	}
 
+	if (iface->MinRtrAdvInterval < 0)
+		iface->MinRtrAdvInterval = DFLT_MinRtrAdvInterval(iface);
+
 	if ((iface->MinRtrAdvInterval < (MIPv6 ? MIN_MinRtrAdvInterval_MIPv6 : MIN_MinRtrAdvInterval)) || 
 		    (iface->MinRtrAdvInterval > MAX_MinRtrAdvInterval(iface)))
 	{
@@ -147,18 +162,25 @@ check_iface(struct Interface *iface)
 			iface->Name, iface->AdvCurHopLimit, MAX_AdvCurHopLimit);
 		res = -1;
 	}
-	
+
+	if (iface->AdvDefaultLifetime < 0)
+		iface->AdvDefaultLifetime = DFLT_AdvDefaultLifetime(iface);
+
 	if ((iface->AdvDefaultLifetime != 0) &&
 	   ((iface->AdvDefaultLifetime > MAX_AdvDefaultLifetime) ||
 	    (iface->AdvDefaultLifetime < MIN_AdvDefaultLifetime(iface))))
 	{
 		flog(LOG_ERR, 
 			"AdvDefaultLifetime for %s (%u) must be zero or between %u and %u",
-			iface->Name, iface->AdvDefaultLifetime, MIN_AdvDefaultLifetime(iface),
+			iface->Name, iface->AdvDefaultLifetime, (int)MIN_AdvDefaultLifetime(iface),
 			MAX_AdvDefaultLifetime);
 		res = -1;
 	}
 
+	/* Mobile IPv6 ext */
+	if (iface->HomeAgentLifetime < 0)
+		iface->HomeAgentLifetime = DFLT_HomeAgentLifetime(iface);
+
 	/* Mobile IPv6 ext */
 	if (iface->AdvHomeAgentInfo)
 	{
